from typing import TypeVar

from ack_product.DescribeACKAvailableResourcesRequestMain import DescribeACKAvailableResourcesRequest
from ack_product.DescribeACKAvailableResourcesResponseMain import DescribeACKAvailableResourcesResponse
from ack_product.DescribeACKPublicImagesRequestMain import DescribeACKPublicImagesRequest
from ack_product.DescribeACKPublicImagesResponseMain import DescribeACKPublicImagesResponse
from ack_product.DescribeACKServiceTypesRequestMain import DescribeACKServiceTypesRequest
from ack_product.DescribeACKServiceTypesResponseMain import DescribeACKServiceTypesResponse
from ack_product.DescribeACKVolumeTypesRequestMain import DescribeACKVolumeTypesRequest
from ack_product.DescribeACKVolumeTypesResponseMain import DescribeACKVolumeTypesResponse
from client.AICloudRestTemplateMain import AICloudRestTemplate
from common.ClientProfileMain import ClientProfile
from common.HttpProfileMain import HttpProfile
from common.constants import AICloudConstants
from common.exception.AICloudExceptionMain import InvalidArgumentException


T = TypeVar('T')


class ACKProductClient(dict):
    httpProfile: HttpProfile
    clientProfile: ClientProfile

    client: AICloudRestTemplate

    def __init__(self, httpProfile: HttpProfile = None, clientProfile: ClientProfile = None):
        self.clientProfile = clientProfile
        if httpProfile is None:
            self.httpProfile = HttpProfile()
        else:
            self.httpProfile = httpProfile
        self.client = AICloudRestTemplate(self.httpProfile, self.clientProfile)

    def GenericRequest(self, request: dict, action: str, clazz: T) -> T:
        if self.httpProfile is None or self.clientProfile is None:
            raise InvalidArgumentException

        url: str = self.httpProfile.protocol + \
                   "://" + \
                   self.httpProfile.host + \
                   ":" + \
                   str(self.httpProfile.port) + \
                   "/" + \
                   self.httpProfile.contextPath + \
                   "/" + \
                   AICloudConstants.VERSION.lower() + \
                   "/" + \
                   AICloudConstants.ACK_PRODUCT_SERVICE.lower() + \
                   "/" + \
                   action
        headers = dict()
        headers[AICloudConstants.X_AIC_ACTION] = action
        headers[AICloudConstants.X_AIC_ACCESSKEY] = self.clientProfile.accessKey
        headers["Host"] = self.httpProfile.host + ":" + str(self.httpProfile.port)
        headers[AICloudConstants.X_AIC_SERVICE] = AICloudConstants.ACK_PRODUCT_SERVICE

        response = self.client.postForResponse(url, headers, body=request)
        jsonObj = response.json()
        return clazz(jsonObj)

    def DescribeACKServiceTypes(self, request: DescribeACKServiceTypesRequest) -> DescribeACKServiceTypesResponse:
        return self.GenericRequest(request.__dict__, "DescribeACKServiceTypes", DescribeACKServiceTypesResponse)

    def DescribeACKVolumeTypes(self, request: DescribeACKVolumeTypesRequest) -> DescribeACKVolumeTypesResponse:
        return self.GenericRequest(request.__dict__, "DescribeACKVolumeTypes", DescribeACKVolumeTypesResponse)

    def DescribeACKPublicImages(self, request: DescribeACKPublicImagesRequest) -> DescribeACKPublicImagesResponse:
        return self.GenericRequest(request.__dict__, "DescribeACKPublicImages", DescribeACKPublicImagesResponse)

    def DescribeACKAvailableResources(self,
                                      request: DescribeACKAvailableResourcesRequest) -> DescribeACKAvailableResourcesResponse:
        return self.GenericRequest(request.__dict__, "DescribeACKAvailableResources",
                                   DescribeACKAvailableResourcesResponse)
