from typing import TypeVar

from ackci.CreateImagesRequestMain import CreateImagesRequest
from ackci.CreateImagesResponseMain import CreateImagesResponse
from ackci.DeleteImagesRequestMain import DeleteImagesRequest
from ackci.DeleteImagesResponseMain import DeleteImagesResponse
from client.AICloudRestTemplateMain import AICloudRestTemplate
from common.ClientProfileMain import ClientProfile
from common.HttpProfileMain import HttpProfile
from common.constants import AICloudConstants
from common.exception.AICloudExceptionMain import InvalidArgumentException
from ackci.DescribeImagesRequestMain import DescribeImagesRequest
from ackci.DescribeImagesResponseMain import DescribeImagesResponse

T = TypeVar('T')


class ACKCIClient(dict):
    httpProfile: HttpProfile
    clientProfile: ClientProfile

    client: AICloudRestTemplate

    def __init__(self, httpProfile: HttpProfile = None, clientProfile: ClientProfile = None):
        self.clientProfile = clientProfile
        if httpProfile is None:
            self.httpProfile = HttpProfile()
        else:
            self.httpProfile = httpProfile
        self.client = AICloudRestTemplate(self.httpProfile, self.clientProfile)

    def GenericRequest(self, request: dict, action: str, clazz: T) -> T:
        if self.httpProfile is None or self.clientProfile is None:
            raise InvalidArgumentException

        url: str = self.httpProfile.protocol + \
                   "://" + \
                   self.httpProfile.host + \
                   ":" + \
                   str(self.httpProfile.port) + \
                   "/" + \
                   self.httpProfile.contextPath + \
                   "/" + \
                   AICloudConstants.VERSION.lower() + \
                   "/" + \
                   AICloudConstants.ACKCI_SERVICE.lower() + \
                   "/" + \
                   action
        headers = dict()
        headers[AICloudConstants.X_AIC_ACTION] = action
        headers[AICloudConstants.X_AIC_ACCESSKEY] = self.clientProfile.accessKey
        headers["Host"] = self.httpProfile.host + ":" + str(self.httpProfile.port)
        headers[AICloudConstants.X_AIC_SERVICE] = AICloudConstants.ACKCI_SERVICE

        response = self.client.postForResponse(url, headers, body=request)
        jsonObj = response.json()
        return clazz(jsonObj)

    def CreateImages(self, request: CreateImagesRequest) -> CreateImagesResponse:
        return self.GenericRequest(request.__dict__, "CreateImages", CreateImagesResponse)

    def DeleteImages(self, request: DeleteImagesRequest) -> DeleteImagesResponse:
        return self.GenericRequest(request.__dict__, "DeleteImages", DeleteImagesResponse)
    def DescribeImages(self, request: DescribeImagesRequest) -> DescribeImagesResponse:
        return self.GenericRequest(request.__dict__, "DescribeImages", DescribeImagesResponse)
