from typing import TypeVar

from ackcs.ChangeServicesBillingTypeRequestMain import ChangeServicesBillingTypeRequest
from ackcs.ChangeServicesBillingTypeResponseMain import ChangeServicesBillingTypeResponse
from ackcs.ChangeServicesExpireStrategyRequestMain import ChangeServicesExpireStrategyRequest
from ackcs.ChangeServicesExpireStrategyResponseMain import ChangeServicesExpireStrategyResponse
from ackcs.ChangeServicesModelRequestMain import ChangeServicesModelRequest
from ackcs.ChangeServicesModelResponseMain import ChangeServicesModelResponse
from ackcs.CreateServicesRequestMain import CreateServicesRequest
from ackcs.CreateServicesResponseMain import CreateServicesResponse
from ackcs.DeleteServicesRequestMain import DeleteServicesRequest
from ackcs.DeleteServicesResponseMain import DeleteServicesResponse
from ackcs.DescribeServicesJupyterRequestMain import DescribeServicesJupyterRequest
from ackcs.DescribeServicesJupyterResponseMain import DescribeServicesJupyterResponse
from ackcs.DescribeServicesRequestMain import DescribeServicesRequest
from ackcs.DescribeServicesResponseMain import DescribeServicesResponse
from ackcs.DescribeServicesSSHRequestMain import DescribeServicesSSHRequest
from ackcs.DescribeServicesSSHResponseMain import DescribeServicesSSHResponse
from ackcs.DescribeServicesTensorBoardRequestMain import DescribeServicesTensorBoardRequest
from ackcs.DescribeServicesTensorBoardResponseMain import DescribeServicesTensorBoardResponse
from ackcs.InquiryPriceCreateServicesResponseMain import InquiryPriceCreateServicesResponse
from ackcs.RebootServicesRequestMain import RebootServicesRequest
from ackcs.RebootServicesResponseMain import RebootServicesResponse
from ackcs.RenewServicesRequestMain import RenewServicesRequest
from ackcs.RenewServicesResponseMain import RenewServicesResponse
from ackcs.StartServicesRequestMain import StartServicesRequest
from ackcs.StartServicesResponseMain import StartServicesResponse
from ackcs.StopServicesRequestMain import StopServicesRequest
from ackcs.StopServicesResponseMain import StopServicesResponse
from ackcv.DeleteVolumesRequestMain import DeleteVolumesRequest
from ackcv.DeleteVolumesResponseMain import DeleteVolumesResponse
from ackcv.DescribeVolumesRequestMain import DescribeVolumesRequest
from ackcv.DescribeVolumesResponseMain import DescribeVolumesResponse
from ackcv.ResizeVolumesRequestMain import ResizeVolumesRequest
from ackcv.ResizeVolumesResponseMain import ResizeVolumesResponse
from client.AICloudRestTemplateMain import AICloudRestTemplate
from common.ClientProfileMain import ClientProfile
from common.HttpProfileMain import HttpProfile
from common.constants import AICloudConstants
from common.exception.AICloudExceptionMain import InvalidArgumentException

T = TypeVar('T')


class ACKCVClient(dict):
    httpProfile: HttpProfile
    clientProfile: ClientProfile

    client: AICloudRestTemplate

    def __init__(self, httpProfile: HttpProfile = None, clientProfile: ClientProfile = None):
        self.clientProfile = clientProfile
        if httpProfile is None:
            self.httpProfile = HttpProfile()
        else:
            self.httpProfile = httpProfile
        self.client = AICloudRestTemplate(self.httpProfile, self.clientProfile)

    def GenericRequest(self, request: dict, action: str, clazz: T) -> T:
        if self.httpProfile is None or self.clientProfile is None:
            raise InvalidArgumentException

        url: str = self.httpProfile.protocol + \
                   "://" + \
                   self.httpProfile.host + \
                   ":" + \
                   str(self.httpProfile.port) + \
                   "/" + \
                   self.httpProfile.contextPath + \
                   "/" + \
                   AICloudConstants.VERSION.lower() + \
                   "/" + \
                   AICloudConstants.ACKCV_SERVICE.lower() + \
                   "/" + \
                   action
        headers = dict()
        headers[AICloudConstants.X_AIC_ACTION] = action
        headers[AICloudConstants.X_AIC_ACCESSKEY] = self.clientProfile.accessKey
        headers["Host"] = self.httpProfile.host + ":" + str(self.httpProfile.port)
        headers[AICloudConstants.X_AIC_SERVICE] = AICloudConstants.ACKCV_SERVICE

        response = self.client.postForResponse(url, headers, body=request)
        jsonObj = response.json()
        return clazz(jsonObj)

    def ResizeVolumes(self, request: ResizeVolumesRequest) -> ResizeVolumesResponse:
        return self.GenericRequest(request.__dict__, "ResizeVolumes", ResizeVolumesResponse)

    def DeleteVolumes(self, request: DeleteVolumesRequest) -> DeleteVolumesResponse:
        return self.GenericRequest(request.__dict__, "DeleteVolumes", DeleteVolumesResponse)

    def DescribeVolumes(self, request: DescribeVolumesRequest) -> DescribeVolumesResponse:
        return self.GenericRequest(request.__dict__, "DescribeVolumes", DescribeVolumesResponse)
