from common.constants.AICloudConstants import SIGNATURE_METHOD_VALUE, VERSION, SIGNING_STRING
from common.utils import SignUtils


class AICloudClientSigner(object):
    accessKey: str = None
    secretKey: str = None
    httpMethod: str = None
    servletPath: str = None
    headers: dict[str, str] = None
    queryParams: dict[str, object] = None
    signedHeaders: str = None
    service: str = None
    requestBody: str = None

    def __init__(self, accessKey: str, secretKey: str, httpMethod: str, servletPath: str, headers: dict[str, str],
                 queryParams: dict[str, object], signedHeaders: str, service: str, requestBody: str):
        self.accessKey = accessKey
        self.secretKey = secretKey
        self.httpMethod = httpMethod
        self.servletPath = servletPath
        self.headers = headers
        self.queryParams = queryParams
        self.signedHeaders = signedHeaders
        self.service = service
        self.requestBody = requestBody

    def getCanonicalRequest(self):
        return SignUtils.CanonicalRequest(
            self.httpMethod,
            self.servletPath,
            self.headers,
            self.signedHeaders,
            self.queryParams,
            self.requestBody
        )

    def getStringToSign(self):
        canonicalRequest = self.getCanonicalRequest()
        return SignUtils.CanonicalStringToSign(
            SIGNATURE_METHOD_VALUE,
            VERSION,
            self.accessKey,
            self.service,
            "paratera/aicloud/" + self.service,
            canonicalRequest
        )

    def doSignature(self) -> str:
        stringToSign = self.getStringToSign()
        return SignUtils.Signature(
            SIGNING_STRING + self.secretKey,
            stringToSign
        )
