# 生成规范的请求查询参数串
import hashlib
import hmac
import urllib.parse
from _sha256 import sha256

from common.constants.AICloudConstants import ENCODING, STRING_SEPARATOR
from common.exception.AICloudExceptionMain import InvalidArgumentException


def CanonicalQueryString(queryParams: dict[str, object]) -> str:
    if queryParams is None:
        return ""


# 生成规范的请求Header
def CanonicalHeaders(headers: dict[str, str], signedHeaders: str) -> str:
    if headers is None or signedHeaders is None:
        raise InvalidArgumentException

    finalHeaders = dict[str, str]()
    for _header in headers.keys():
        finalHeaders[str(_header).lower()] = headers.get(_header)

    headerNames = list(finalHeaders.keys())
    if not headerNames.__contains__("host") or not headerNames.__contains__("content-type"):
        raise InvalidArgumentException

    if not signedHeaders.__contains__(";") or not signedHeaders.__contains__("host") or not signedHeaders.__contains__(
            "content-type"):
        raise InvalidArgumentException

    ret = str()
    sh = signedHeaders.split(";")
    shc = len(sh)
    for i in range(shc):
        _sh = sh[i]
        _sv = finalHeaders.get(_sh)
        if "host" == _sh:
            ret = ret + _sh + ":" + getHost(_sv)
        else:
            ret = ret + _sh + ":" + _sv
        if i < shc - 1:
            ret = ret + STRING_SEPARATOR
    return ret


# 生成规范的请求体
def HashedRequestPayload(payload: str) -> str:
    return encode(payload)


# 生成规范的请求字符串
def CanonicalRequest(httpRequestMethod: str, servletPath: str, headers: dict[str, str], signedHeaders: str,
                     queryParams: dict[str, object],
                     payload: str) -> str:
    return httpRequestMethod.upper() + \
           STRING_SEPARATOR + \
           servletPath + \
           STRING_SEPARATOR + \
           CanonicalQueryString(queryParams) + \
           STRING_SEPARATOR + \
           CanonicalHeaders(headers, signedHeaders) + \
           STRING_SEPARATOR + \
           signedHeaders + \
           STRING_SEPARATOR + \
           HashedRequestPayload(payload)


def getHost(hostHeader: str) -> str:
    if ":" not in hostHeader:
        return hostHeader
    else:
        return hostHeader.split(":")[0]


# 生成待签名字符串
def CanonicalStringToSign(algorithm: str, version: str, accessKey: str, service: str, scope: str,
                          canonicalRequest: str) -> str:
    return algorithm + \
           STRING_SEPARATOR + \
           version + \
           STRING_SEPARATOR + \
           accessKey + \
           STRING_SEPARATOR + \
           service + \
           STRING_SEPARATOR + \
           scope + \
           STRING_SEPARATOR + \
           encode(canonicalRequest)


# 查询参数编码
def percentEncode(value: str) -> str:
    return urllib.parse.urlencode({"name": value}).split("=")[1]


# 计算签名
def Signature(secretKey: str, data: str) -> str:
    key = secretKey.encode('utf-8')
    message = data.encode('utf-8')
    sign = hmac.new(key, message, digestmod=sha256).digest()
    return sign.hex()


# 字符串转换成SHA-256编码
def encode(data: str) -> str:
    return hashlib.sha256(data.encode(ENCODING)).hexdigest()


# 对字符串进行16进制编码
def encodeHex(data: bytes) -> str:
    return data.hex()
