from typing import TypeVar

from client.AICloudRestTemplateMain import AICloudRestTemplate
from common.ClientProfileMain import ClientProfile
from common.HttpProfileMain import HttpProfile
from common.constants import AICloudConstants
from common.exception.AICloudExceptionMain import InvalidArgumentException
from ebs.AttachDisksRequestMain import AttachDisksRequest
from ebs.AttachDisksResponseMain import AttachDisksResponse
from ebs.CreateDisksRequestMain import CreateDisksRequest
from ebs.CreateDisksResponseMain import CreateDisksResponse
from ebs.DeleteDisksRequestMain import DeleteDisksRequest
from ebs.DeleteDisksResponseMain import DeleteDisksResponse
from ebs.DescribeDisksRequestMain import DescribeDisksRequest
from ebs.DescribeDisksResponseMain import DescribeDisksResponse
from ebs.DetachDisksRequestMain import DetachDisksRequest
from ebs.DetachDisksResponseMain import DetachDisksResponse
from ebs.InquiryPriceCreateDisksResponseMain import InquiryPriceCreateDisksResponse
from ebs.RenewDisksRequestMain import RenewDisksRequest
from ebs.RenewDisksResponseMain import RenewDisksResponse

T = TypeVar('T')


class EBSClient(dict):
    httpProfile: HttpProfile
    clientProfile: ClientProfile

    client: AICloudRestTemplate

    def __init__(self, httpProfile: HttpProfile = None, clientProfile: ClientProfile = None):
        self.clientProfile = clientProfile
        if httpProfile is None:
            self.httpProfile = HttpProfile()
        else:
            self.httpProfile = httpProfile
        self.client = AICloudRestTemplate(self.httpProfile, self.clientProfile)

    def GenericRequest(self, request: dict, action: str, clazz: T) -> T:
        if self.httpProfile is None or self.clientProfile is None:
            raise InvalidArgumentException

        url: str = self.httpProfile.protocol + \
                   "://" + \
                   self.httpProfile.host + \
                   ":" + \
                   str(self.httpProfile.port) + \
                   "/" + \
                   self.httpProfile.contextPath + \
                   "/" + \
                   AICloudConstants.VERSION.lower() + \
                   "/" + \
                   AICloudConstants.EBS_SERVICE.lower() + \
                   "/" + \
                   action
        headers = dict()
        headers[AICloudConstants.X_AIC_ACTION] = action
        headers[AICloudConstants.X_AIC_ACCESSKEY] = self.clientProfile.accessKey
        # headers["Host"] = self.httpProfile.host + ":" + self.httpProfile.port
        headers["Host"] = self.httpProfile.host + ":" + str(self.httpProfile.port)
        headers[AICloudConstants.X_AIC_SERVICE] = AICloudConstants.EBS_SERVICE

        response = self.client.postForResponse(url, headers, body=request)
        jsonObj = response.json()
        return clazz(jsonObj)

    def InquiryPriceCreateDisks(self, request: CreateDisksRequest) -> InquiryPriceCreateDisksResponse:
        return self.GenericRequest(request.__dict__, "InquiryPriceCreateDisks", InquiryPriceCreateDisksResponse)

    def DescribeDisks(self, request: DescribeDisksRequest) -> DescribeDisksResponse:
        if request.pageNum is not None:
            if request.pageNum <= 0:
                raise InvalidArgumentException
        return self.GenericRequest(request.__dict__, "DescribeDisks", DescribeDisksResponse)

    def CreateDisks(self, request: CreateDisksRequest) -> CreateDisksResponse:
        return self.GenericRequest(request.__dict__, "CreateDisks", CreateDisksResponse)

    def AttachDisks(self, request: AttachDisksRequest) -> AttachDisksResponse:
        return self.GenericRequest(request.__dict__, "AttachDisks", AttachDisksResponse)

    def DetachDisks(self, request: DetachDisksRequest) -> DetachDisksResponse:
        return self.GenericRequest(request.__dict__, "DetachDisks", DetachDisksResponse)

    def DeleteDisks(self, request: DeleteDisksRequest) -> DeleteDisksResponse:
        return self.GenericRequest(request.__dict__, "DeleteDisks", DeleteDisksResponse)

    def RenewDisks(self, request: RenewDisksRequest) -> RenewDisksResponse:
        return self.GenericRequest(request.__dict__, "RenewDisks", RenewDisksResponse)
