from unittest import TestCase

from common.ClientProfileMain import ClientProfile
from common.HttpProfileMain import HttpProfile
from common.enums.BillingTypeEnum import BillingTypeEnum
from ebs.AttachDisksRequestMain import AttachDisksRequest
from ebs.CreateDisksRequestMain import CreateDisksRequest
from ebs.DeleteDisksRequestMain import DeleteDisksRequest
from ebs.DescribeDisksRequestMain import DescribeDisksRequest
from ebs.DetachDisksRequestMain import DetachDisksRequest
from ebs.EBSClientMain import EBSClient
from ebs.RenewDisksRequestMain import RenewDisksRequest


class TestCBS(TestCase):

    def test_InquiryPrice(self):
        httpProfile = HttpProfile(
            protocol="http",
            host="127.0.0.1",
            port=8899
        )
        clientProfile = ClientProfile(
            accessKey="abcd",
            secretAccessKey="abcd1234"
        )

        client = EBSClient(httpProfile=httpProfile, clientProfile=clientProfile)
        request = CreateDisksRequest()
        request.zoneCode = "cn-zhongwei-a"
        request.ebsName = "ebs-abcdeff"
        request.billingType = BillingTypeEnum.PostPaid.name
        request.diskType = "HDD"
        request.diskSize = 30
        request.payPeriod = 0
        response = client.InquiryPriceCreateDisks(request)

        code = response.code
        message = response.message
        if code != 200:
            print(message)
            return None

        data = response.data
        if data is not None:
            print(data.__dict__)
            items = data.items
            if items is not None:
                for item in items:
                    print(item.__dict__)

    def test_CreateDisks(self):
        httpProfile = HttpProfile(
            protocol="http",
            host="127.0.0.1",
            port=8899
        )
        clientProfile = ClientProfile(
            accessKey="abcd",
            secretAccessKey="abcd1234"
        )

        client = EBSClient(httpProfile=httpProfile, clientProfile=clientProfile)
        request = CreateDisksRequest()
        request.zoneCode = "cn-zhongwei-a"
        request.aliasName = "ebs-abcdeff"
        request.billingType = BillingTypeEnum.PrePaid.name
        request.diskType = "HDD"
        request.diskSize = 30
        request.payPeriod = 1
        response = client.CreateDisks(request)

        code = response.code
        message = response.message
        if code != 200:
            print(message)
            return None

        jobs = response.data
        if jobs is not None:
            for job in jobs:
                print(job.__dict__)
    def test_AttachDisks(self):
        httpProfile = HttpProfile(
            protocol="http",
            host="127.0.0.1",
            port=8899
        )
        clientProfile = ClientProfile(
            accessKey="abcd",
            secretAccessKey="abcd1234"
        )

        client = EBSClient(httpProfile=httpProfile, clientProfile=clientProfile)
        request = AttachDisksRequest()
        request.zoneCode = "cn-zhongwei-a"
        request.ebsUuids = ["7334cd8a-bdc9-4087-af5b-b643fdec6800"]
        request.ecsUuid = "1087c175-d996-4715-8915-1ac4a1159667"
        response = client.AttachDisks(request)

        code = response.code
        message = response.message
        if code != 200:
            print(message)
            return None

        jobs = response.data
        if jobs is not None:
            for job in jobs:
                print(job.__dict__)

    def test_DetachDisks(self):
        httpProfile = HttpProfile(
            protocol="http",
            host="127.0.0.1",
            port=8899
        )
        clientProfile = ClientProfile(
            accessKey="abcd",
            secretAccessKey="abcd1234"
        )

        client = EBSClient(httpProfile=httpProfile, clientProfile=clientProfile)
        request = DetachDisksRequest()
        request.zoneCode = "cn-zhongwei-a"
        request.ebsUuids = ["7334cd8a-bdc9-4087-af5b-b643fdec6800"]
        request.ecsUuid = "1087c175-d996-4715-8915-1ac4a1159667"
        response = client.DetachDisks(request)

        code = response.code
        message = response.message
        if code != 200:
            print(message)
            return None
        jobs = response.data
        if jobs is not None:
            for job in jobs:
                print(job.__dict__)
        return None


    def test_DeleteDisks(self):
        httpProfile = HttpProfile(
            protocol="http",
            host="127.0.0.1",
            port=8899
        )
        clientProfile = ClientProfile(
            accessKey="abcd",
            secretAccessKey="abcd1234"
        )

        client = EBSClient(httpProfile=httpProfile, clientProfile=clientProfile)
        request = DeleteDisksRequest()
        request.zoneCode = "cn-zhongwei-a"
        request.ebsUuids = ["d4c4b93b-3e6c-47b0-b8c8-eba08f40b1c8"]
        response = client.DeleteDisks(request)

        code = response.code
        message = response.message
        if code != 200:
            print(message)
            return None

        jobs = response.data
        if jobs is not None:
            for job in jobs:
                print(job.__dict__)
        return None


    def test_RenewDisks(self):
        httpProfile = HttpProfile(
            protocol="http",
            host="127.0.0.1",
            port=8899
        )
        clientProfile = ClientProfile(
            accessKey="abcd",
            secretAccessKey="abcd1234"
        )

        client = EBSClient(httpProfile=httpProfile, clientProfile=clientProfile)
        request = RenewDisksRequest()
        request.zoneCode = "cn-zhongwei-a"
        request.ebsUuids = ["84bb9b4e-e59a-4c9c-8d1d-26c47e7e2792"]
        request.period = 1
        response = client.RenewDisks(request)

        code = response.code
        message = response.message
        if code != 200:
            print(message)
            return None


    def test_DescribeDisks(self):
        httpProfile = HttpProfile(
            protocol="http",
            host="127.0.0.1",
            port=8899
        )
        clientProfile = ClientProfile(
            accessKey="abcd",
            secretAccessKey="abcd1234"
        )

        client = EBSClient(httpProfile=httpProfile, clientProfile=clientProfile)
        request = DescribeDisksRequest()
        # request.ecsUuid = "de5995e8-180d-4445-842f-1e3ab0d15d3a"
        # request.ebsUuid = "9d0a06b8-cd8a-4bd5-af8e-445b8b63b0e7"
        request.pageNum = 2
        request.pageSize = 200
        response = client.DescribeDisks(request)

        code = response.code
        message = response.message
        if code != 200:
            print(message)
            return None

        pageData = response.data
        if pageData is not None:
            print(pageData.__dict__)
            rows = pageData.rows
            if rows is not None:
                for row in rows:
                    print(row.__dict__)
        return None
