import unittest

from common.ClientProfileMain import ClientProfile
from common.EBSInputMain import EBSInput
from common.EIPInputMain import EIPInput
from common.HttpProfileMain import HttpProfile
from common.enums import BillingTypeEnum
from common.enums.StopTypeEnum import StopTypeEnum
from common.enums.StoppedModeEnum import StoppedModeEnum
from ecs.DeleteInstancesRequestMain import DeleteInstancesRequest
from ecs.DescribeInstancesRequestMain import DescribeInstancesRequest
from ecs.ECSClientMain import ECSClient
from ecs.RebootInstancesRequestMain import RebootInstancesRequest
from ecs.RenewInstancesRequestMain import RenewInstancesRequest
from ecs.RunInstancesRequestMain import RunInstancesRequest
from ecs.StartInstancesRequestMain import StartInstancesRequest
from ecs.StopInstancesRequestMain import StopInstancesRequest


class MyTestCase(unittest.TestCase):

    def test_InquiryPrice(self):
        httpProfile = HttpProfile(
            protocol="http",
            host="127.0.0.1",
            port=8899
        )
        clientProfile = ClientProfile(
            accessKey="abcd",
            secretAccessKey="abcd1234"
        )

        client = ECSClient(httpProfile=httpProfile, clientProfile=clientProfile)
        request = RunInstancesRequest()
        request.zoneCode = "cn-zhongwei-a"
        request.count = 1
        request.ecsModel = "ecs.cpu.calc.4c8g"
        request.imageUuid = "636bc998-9382-47e9-9573-6dbac34abda7"
        request.rootDiskType = "HDD"
        request.rootDiskSize = 20
        request.billingType = BillingTypeEnum.BillingTypeEnum.PostPaid.name
        request.payPeriod = 0
        ebs = EBSInput({"diskType": "HDD", "diskSize": "30"})
        request.ebs = [ebs]
        eip = EIPInput({"networkSize": 10, "networkType": "BandWidth"})
        request.eip = eip
        request.autoContinue = False
        response = client.InquiryPriceRunInstances(request)
        code = response.code
        message = response.message
        if code != 200:
            print(message)
            return None

        data = response.data
        if data is not None:
            print(data.__dict__)
            items = data.items
            if items is not None:
                for item in items:
                    print(item.__dict__)


    def test_RunInstances(self):
        httpProfile = HttpProfile(
            protocol="http",
            host="127.0.0.1",
            port=8899
        )
        clientProfile = ClientProfile(
            accessKey="abcd",
            secretAccessKey="abcd1234"
        )

        client = ECSClient(httpProfile=httpProfile, clientProfile=clientProfile)
        request = RunInstancesRequest()
        request.zoneCode = "cn-zhongwei-a"
        request.count = 1
        request.ecsModel = "ecs.cpu.calc.4c8g"
        request.aliasName = "ecs-abcdefg56"
        request.imageUuid = "7837cb93-ec27-4f19-a273-0ee3d2a681cf"
        request.rootDiskType = "HDD"
        request.rootDiskSize = 20
        request.billingType = BillingTypeEnum.BillingTypeEnum.PrePaid.name
        request.payPeriod=1
        ebs = EBSInput({"diskType": "HDD", "diskSize": "30"})
        request.ebs = [ebs]
        eip = EIPInput({"networkSize": 10, "networkType": "Flow"})
        request.eip = eip
        request.autoContinue = False
        request.password="3165666e50492e67464e616c49614155"
        response = client.RunInstances(request)
        code = response.code
        message = response.message
        if code != 200:
            print(message)
            return None

        jobs = response.data
        if jobs is not None:
            for job in jobs:
                print(job.__dict__)

    def test_StartInstances(self):
        httpProfile = HttpProfile(
            protocol="http",
            host="127.0.0.1",
            port=8899
        )
        clientProfile = ClientProfile(
            accessKey="abcd",
            secretAccessKey="abcd1234"
        )

        client = ECSClient(httpProfile=httpProfile, clientProfile=clientProfile)
        request = StartInstancesRequest()
        request.zoneCode = "cn-zhongwei-a"
        request.ecsUuids = ["56f0f968-f175-48b5-91cb-378cfb688402"]
        response = client.StartInstances(request)

        code = response.code
        message = response.message
        if code != 200:
            print(message)
            return None

        jobs = response.data
        if jobs is not None:
            for job in jobs:
                print(job.__dict__)
        return None

    def test_StopInstances(self):
        httpProfile = HttpProfile(
            protocol="http",
            host="127.0.0.1",
            port=8899
        )
        clientProfile = ClientProfile(
            accessKey="abcd",
            secretAccessKey="abcd1234"
        )

        client = ECSClient(httpProfile=httpProfile, clientProfile=clientProfile)
        request = StopInstancesRequest()
        request.zoneCode = "cn-zhongwei-a"
        request.ecsUuids = ["56f0f968-f175-48b5-91cb-378cfb688402"]
        request.stopType = StopTypeEnum.SOFT.name
        request.stoppedMode = StoppedModeEnum.KEEP_CHARGING.name
        request.releaseEip = False
        response = client.StopInstances(request)

        code = response.code
        message = response.message
        if code != 200:
            print(message)
            return None

        jobs = response.data
        if jobs is not None:
            for job in jobs:
                print(job.__dict__)

        return None

    def test_RebootInstances(self):
        httpProfile = HttpProfile(
            protocol="http",
            host="127.0.0.1",
            port=8899
        )
        clientProfile = ClientProfile(
            accessKey="abcd",
            secretAccessKey="abcd1234"
        )

        client = ECSClient(httpProfile=httpProfile, clientProfile=clientProfile)
        request = RebootInstancesRequest()
        request.zoneCode = "cn-zhongwei-a"
        request.ecsUuids = ["56f0f968-f175-48b5-91cb-378cfb688402"]
        request.stopType = StopTypeEnum.SOFT.name
        response = client.RebootInstances(request)

        code = response.code
        message = response.message
        if code != 200:
            print(message)
            return None

        jobs = response.data
        if jobs is not None:
            for job in jobs:
                print(job.__dict__)
        return None

    def test_DeleteInstances(self):
        httpProfile = HttpProfile(
            protocol="http",
            host="127.0.0.1",
            port=8899
        )
        clientProfile = ClientProfile(
            accessKey="abcd",
            secretAccessKey="abcd1234"
        )

        client = ECSClient(httpProfile=httpProfile, clientProfile=clientProfile)

        request = DeleteInstancesRequest()
        request.zoneCode = "cn-zhongwei-a"
        request.ecsUuids = ["56f0f968-f175-48b5-91cb-378cfb688402"]
        request.deleteEip = True
        request.deleteEbs = True
        response = client.DeleteInstance(request)

        code = response.code
        message = response.message
        if code != 200:
            print(message)
            return None
        jobs = response.data
        if jobs is not None:
            for job in jobs:
                print(job.__dict__)
        return None

    def test_RenewInstances(self):
        httpProfile = HttpProfile(
            protocol="http",
            host="127.0.0.1",
            port=8899
        )
        clientProfile = ClientProfile(
            accessKey="abcd",
            secretAccessKey="abcd1234"
        )

        client = ECSClient(httpProfile=httpProfile, clientProfile=clientProfile)

        request = RenewInstancesRequest()
        request.zoneCode = "cn-zhongwei-a"
        request.ecsUuids = ["56f0f968-f175-48b5-91cb-378cfb688402"]
        request.period = 1
        response = client.RenewInstances(request)

        code = response.code
        message = response.message
        if code != 200:
            print(message)
            return None
        return None

    def test_DescribeInstances(self):
        httpProfile = HttpProfile(
            protocol="http",
            host="127.0.0.1",
            port=8899
        )
        clientProfile = ClientProfile(
            accessKey="abcd",
            secretAccessKey="abcd1234"
        )

        client = ECSClient(httpProfile=httpProfile, clientProfile=clientProfile)
        request = DescribeInstancesRequest()
        # request.eip = "130.234.213.23"
        # request.ip = "172.18.2.23"
        # request.ecsUuid = "a5fd0143-0a6b-4825-b26a-142e2367c560"
        request.pageNum = 1
        request.pageSize = 200
        response = client.DescribeInstances(request)

        code = response.code
        message = response.message
        if code != 200:
            print(message)
            return None

        pageDatas = response.data
        if pageDatas is not None:
            print(pageDatas.__dict__)
            rows = pageDatas.rows
            if rows is not None:
                for row in rows:
                    print(row.__dict__)


        return None


if __name__ == '__main__':
    unittest.main()
