from typing import TypeVar

from client.AICloudRestTemplateMain import AICloudRestTemplate
from common.ClientProfileMain import ClientProfile
from common.HttpProfileMain import HttpProfile
from common.PartnerClientProfileMain import KeyClientProfile
from common.constants import AICloudConstants
from common.exception.AICloudExceptionMain import InvalidArgumentException
from key.CreateAPIKeyRequestMain import CreateAPIKeyRequest
from key.CreateAPIKeyResponseMain import CreateAPIKeyResponse
from key.DeleteAPIKeyRequestMain import DeleteAPIKeyRequest
from key.DeleteAPIKeyResponseMain import DeleteAPIKeyResponse
from key.DescribeAPIKeyRequestMain import DescribeAPIKeysRequest
from key.DescribeAPIKeysResponseMain import DescribeAPIKeysResponse
from key.UpdateAPIKeyResponseMain import UpdateAPIKeyResponse

T = TypeVar('T')


class KeyClient(dict):
    httpProfile: HttpProfile
    clientProfile: KeyClientProfile

    client: AICloudRestTemplate

    def __init__(self, httpProfile: HttpProfile = None, clientProfile: ClientProfile = None):
        self.clientProfile = clientProfile
        if httpProfile is None:
            self.httpProfile = HttpProfile()
        else:
            self.httpProfile = httpProfile
        self.client = AICloudRestTemplate(self.httpProfile, self.clientProfile)

    def GenericRequest(self, request: dict, action: str, clazz: T) -> T:
        if self.httpProfile is None or self.clientProfile is None:
            raise InvalidArgumentException

        url: str = self.httpProfile.protocol + \
                   "://" + \
                   self.httpProfile.host + \
                   ":" + \
                   str(self.httpProfile.port) + \
                   "/" + \
                   self.httpProfile.contextPath + \
                   "/" + \
                   AICloudConstants.VERSION.lower() + \
                   "/" + \
                   AICloudConstants.KEY_SERVICE.lower() + \
                   "/" + \
                   action
        headers = dict()
        headers[AICloudConstants.X_AIC_ACTION] = action
        headers[AICloudConstants.X_AIC_ACCESSKEY] = self.clientProfile.accessKey
        headers["Host"] = self.httpProfile.host + ":" + str(self.httpProfile.port)
        headers[AICloudConstants.X_AIC_SERVICE] = AICloudConstants.KEY_SERVICE
        headers[AICloudConstants.X_AIC_PARTNER_ACCESSKEY] = self.clientProfile.partnerKey

        response = self.client.postForResponse(url, headers, body=request)
        jsonObj = response.json()
        return clazz(jsonObj)

    def CreateAPIKey(self, request: CreateAPIKeyRequest) -> CreateAPIKeyResponse:
        return self.GenericRequest(request.__dict__, "CreateAPIKey", CreateAPIKeyResponse)
    def DeleteAPIKey(self, request: DeleteAPIKeyRequest) -> DeleteAPIKeyResponse:
        return self.GenericRequest(request.__dict__, "DeleteAPIKey", DeleteAPIKeyResponse)
    def UpdateAPIKey(self, request: CreateAPIKeyRequest) -> UpdateAPIKeyResponse:
        return self.GenericRequest(request.__dict__, "UpdateAPIKey", UpdateAPIKeyResponse)
    def DescribeAPIKeys(self, request: DescribeAPIKeysRequest) -> DescribeAPIKeysResponse:
        return self.GenericRequest(request.__dict__, "DescribeAPIKeys", DescribeAPIKeysResponse)
