import unittest

from common.HttpProfileMain import HttpProfile
from common.PartnerClientProfileMain import KeyClientProfile
from common.constants import KeyConstants
from key.CreateAPIKeyRequestMain import CreateAPIKeyRequest
from key.DeleteAPIKeyRequestMain import DeleteAPIKeyRequest
from key.DescribeAPIKeyRequestMain import DescribeAPIKeysRequest
from key.KeyClientMain import KeyClient
from key.UpdateAPIKeyRequestMain import UpdateAPIKeyRequest


class MyTestCase(unittest.TestCase):

    def test_CreateAPIKey(self):
        # 创建Http配置参数
        httpProfile = HttpProfile(
            protocol="http",
            host="127.0.0.1",
            port=8899
        )
        # 创建客户端配置参数
        clientProfile = KeyClientProfile(
            accessKey="abcd",
            secretAccessKey="abcd1234",
            partnerKey="Tmc1TDhPV1h5OTRDak91TzZaa3lLaDA5SUFFSjhTejJjVHhqK01IMXQ5NlltSFZHZXl6S3RHeEZUMEtGVFRHNg==",
        )

        # 创建客户端
        client = KeyClient(httpProfile=httpProfile, clientProfile=clientProfile)

        # 创建请求
        request = CreateAPIKeyRequest()

        # 创建API秘钥
        createAPIKeyResponse = client.CreateAPIKey(request)

        if createAPIKeyResponse.code == 200:
            print("创建成功")
        else:
            print(createAPIKeyResponse.message)

    def test_DeleteAPIKey(self):
        # 创建Http配置参数
        httpProfile = HttpProfile(
            protocol="http",
            host="127.0.0.1",
            port=8899
        )

        # 创建客户端配置参数
        clientProfile = KeyClientProfile(
            accessKey="abcd",
            secretAccessKey="abcd1234",
            partnerKey="Tmc1TDhPV1h5OTRDak91TzZaa3lLaDA5SUFFSjhTejJjVHhqK01IMXQ5NlltSFZHZXl6S3RHeEZUMEtGVFRHNg==",
        )

        # 创建客户端
        client = KeyClient(httpProfile=httpProfile, clientProfile=clientProfile)

        # 创建请求
        request = DeleteAPIKeyRequest()
        request.accessKey = "FYcL6J/0xP1H/k1j2Ch/NaFsjZ/AUMrDQDdNqKq513c="

        # 删除秘钥
        deleteAPIKeyResponse = client.DeleteAPIKey(request)

        if deleteAPIKeyResponse.code == 200:
            print("删除成功")
        else:
            print(deleteAPIKeyResponse.message)

    def test_UpdateAPIKey(self):

        # 创建Http配置参数
        httpProfile = HttpProfile(
            protocol="http",
            host="127.0.0.1",
            port=8899
        )

        # 创建客户端配置参数
        clientProfile = KeyClientProfile(
            accessKey="abcd",
            secretAccessKey="abcd1234",
            partnerKey="Tmc1TDhPV1h5OTRDak91TzZaa3lLaDA5SUFFSjhTejJjVHhqK01IMXQ5NlltSFZHZXl6S3RHeEZUMEtGVFRHNg==",
        )

        # 创建客户端
        client = KeyClient(httpProfile=httpProfile, clientProfile=clientProfile)

        # 创建请求
        request = UpdateAPIKeyRequest()
        request.accessKey = "FYcL6J/0xP1H/k1j2Ch/NaFsjZ/AUMrDQDdNqKq513c="
        request.status = KeyConstants.DISABLED

        # 更新秘钥
        updateAPIKeyResponse = client.UpdateAPIKey(request)
        if updateAPIKeyResponse.code == 200:
            print("更新成功")
        else:
            print(updateAPIKeyResponse.message)

    def test_DescribeAPIKeys(self):

        # 创建Http配置参数
        httpProfile = HttpProfile(
            protocol="http",
            host="127.0.0.1",
            port=8899
        )

        # 创建客户端配置参数
        clientProfile = KeyClientProfile(
            accessKey="abcd",
            secretAccessKey="abcd1234",
            partnerKey="Tmc1TDhPV1h5OTRDak91TzZaa3lLaDA5SUFFSjhTejJjVHhqK01IMXQ5NlltSFZHZXl6S3RHeEZUMEtGVFRHNg==",
        )

        # 创建客户端
        client = KeyClient(httpProfile=httpProfile, clientProfile=clientProfile)
        request = DescribeAPIKeysRequest()
        # request.accessKey = "FYcL6J/0xP1H/k1j2Ch/NaFsjZ/AUMrDQDdNqKq513c="

        # 查询秘钥列表
        describeAPIKeyResponse = client.DescribeAPIKeys(request)

        if describeAPIKeyResponse.code == 200:
            for item in describeAPIKeyResponse.data:
                print(item.__dict__)
        else:
            print(describeAPIKeyResponse.message)


if __name__ == '__main__':
    unittest.main()
