from typing import TypeVar

from client.AICloudRestTemplateMain import AICloudRestTemplate
from common.ClientProfileMain import ClientProfile
from common.HttpProfileMain import HttpProfile
from common.constants import AICloudConstants
from common.exception.AICloudExceptionMain import InvalidArgumentException
from product.DescribeDiskTypesRequestMain import DescribeDiskTypesRequest
from product.DescribeDiskTypesResponseMain import DescribeDiskTypesResponse
from product.DescribeImagesRequestMain import DescribeImagesRequest
from product.DescribeImagesResponseMain import DescribeImagesResponse
from product.DescribeInstanceTypesRequestMain import DescribeInstanceTypesRequest
from product.DescribeInstanceTypesResponseMain import DescribeInstanceTypesResponse
from product.DescribeNetworkTypesRequestMain import DescribeNetworkTypesRequest
from product.DescribeNetworkTypesResponseMain import DescribeNetworkTypesResponse

T = TypeVar('T')


class ProductClient(dict):
    httpProfile: HttpProfile
    clientProfile: ClientProfile

    client: AICloudRestTemplate

    def __init__(self, httpProfile: HttpProfile = None, clientProfile: ClientProfile = None):
        self.clientProfile = clientProfile
        if httpProfile is None:
            self.httpProfile = HttpProfile()
        else:
            self.httpProfile = httpProfile
        self.client = AICloudRestTemplate(self.httpProfile, self.clientProfile)

    def GenericRequest(self, request: dict, action: str, clazz: T) -> T:
        if self.httpProfile is None or self.clientProfile is None:
            raise InvalidArgumentException

        url: str = self.httpProfile.protocol + \
                   "://" + \
                   self.httpProfile.host + \
                   ":" + \
                   str(self.httpProfile.port) + \
                   "/" + \
                   self.httpProfile.contextPath + \
                   "/" + \
                   AICloudConstants.VERSION.lower() + \
                   "/" + \
                   AICloudConstants.PRODUCT_SERVICE.lower() + \
                   "/" + \
                   action
        headers = dict()
        headers[AICloudConstants.X_AIC_ACTION] = action
        headers[AICloudConstants.X_AIC_ACCESSKEY] = self.clientProfile.accessKey
        headers["Host"] = self.httpProfile.host + ":" + str(self.httpProfile.port)
        headers[AICloudConstants.X_AIC_SERVICE] = AICloudConstants.PRODUCT_SERVICE

        response = self.client.postForResponse(url, headers, body=request)
        jsonObj = response.json()
        return clazz(jsonObj)

    def DescribeInstanceTypes(self, request: DescribeInstanceTypesRequest) -> DescribeInstanceTypesResponse:
        return self.GenericRequest(request.__dict__, "DescribeInstanceTypes", DescribeInstanceTypesResponse)

    def DescribeDiskTypes(self, request: DescribeDiskTypesRequest) -> DescribeDiskTypesResponse:
        return self.GenericRequest(request.__dict__, "DescribeDiskTypes", DescribeDiskTypesResponse)

    def DescribeNetworkTypes(self, request: DescribeNetworkTypesRequest) -> DescribeNetworkTypesResponse:
        return self.GenericRequest(request.__dict__, "DescribeNetworkTypes", DescribeNetworkTypesResponse)

    def DescribeImages(self, request: DescribeImagesRequest) -> DescribeImagesResponse:
        return self.GenericRequest(request.__dict__, "DescribeImages", DescribeImagesResponse)
