import json
import sys
import unittest

from common.ClientProfileMain import ClientProfile
from common.HttpProfileMain import HttpProfile
from product.DescribeDiskTypesRequestMain import DescribeDiskTypesRequest
from product.DescribeImagesRequestMain import DescribeImagesRequest
from product.DescribeInstanceTypesRequestMain import DescribeInstanceTypesRequest
from product.DescribeNetworkTypesRequestMain import DescribeNetworkTypesRequest
from product.ProductClientMain import ProductClient


class MyTestCase(unittest.TestCase):

    def test_DescribeInstanceTypes(self):
        httpProfile = HttpProfile(
            protocol="http",
            host="127.0.0.1",
            port=8899
        )
        clientProfile = ClientProfile(
            accessKey="abcd",
            secretAccessKey="abcd1234"
        )
        client = ProductClient(httpProfile=httpProfile, clientProfile=clientProfile)

        request = DescribeInstanceTypesRequest()
        response = client.DescribeInstanceTypes(request)
        code = response.code
        message = response.message
        if code != 200:
            print(message)
            sys.exit(0)

        instanceTypes = response.data
        for instanceType in instanceTypes:
            print(instanceType.__dict__)



    def test_DescribeDiskTypes(self):
        httpProfile = HttpProfile(
            protocol="http",
            host="127.0.0.1",
            port=8899
        )
        clientProfile = ClientProfile(
            "abcd",
            "abcd1234"
        )
        client = ProductClient(httpProfile, clientProfile)

        request = DescribeDiskTypesRequest()
        response = client.DescribeDiskTypes(request)
        code = response.code
        message = response.message
        if code != 200:
            print(message)
            return None

        diskTypes = response.data
        if diskTypes is not None:
            for diskType in diskTypes:
                print(diskType.__dict__)

    def test_DescribeNetworkTypes(self):
        httpProfile = HttpProfile(
            protocol="http",
            host="127.0.0.1",
            port=8899
        )
        clientProfile = ClientProfile(
            accessKey="abcd",
            secretAccessKey="abcd1234"
        )
        client = ProductClient(httpProfile=httpProfile, clientProfile=clientProfile)

        request = DescribeNetworkTypesRequest()
        response = client.DescribeNetworkTypes(request)
        code = response.code
        message = response.message
        if code != 200:
            print(message)
            return None

        networkTypes = response.data
        if networkTypes is not None:
            for networkType in networkTypes:
                print(networkType.__dict__)

    def test_DescribeImages(self):
        httpProfile = HttpProfile(
            protocol="http",
            host="127.0.0.1",
            port=8899
        )
        clientProfile = ClientProfile(
            "abcd",
            "abcd1234"
        )
        client = ProductClient(httpProfile, clientProfile)

        request = DescribeImagesRequest()
        response = client.DescribeImages(request)
        code = response.code
        message = response.message
        if code != 200:
            print(message)
            return None

        images = response.data
        if images is not None:
            for image in images:
                print(image.__dict__)


if __name__ == '__main__':
    unittest.main()
