from typing import TypeVar

from client.AICloudRestTemplateMain import AICloudRestTemplate
from common.ClientProfileMain import ClientProfile
from common.HttpProfileMain import HttpProfile
from common.constants import AICloudConstants
from common.exception.AICloudExceptionMain import InvalidArgumentException
from region.DescribeZonesRequestMain import DescribeZonesRequest
from region.DescribeZonesResponseMain import DescribeZonesResponse

T = TypeVar('T')


class RegionClient(object):
    httpProfile: HttpProfile
    clientProfile: ClientProfile

    client: AICloudRestTemplate

    def __init__(self, httpProfile: HttpProfile = None, clientProfile: ClientProfile = None):
        self.clientProfile = clientProfile
        if httpProfile is None:
            self.httpProfile = HttpProfile()
        else:
            self.httpProfile = httpProfile
        self.client = AICloudRestTemplate(self.httpProfile, self.clientProfile)

    def GenericRequest(self, request: dict, action: str, clazz: T) -> T:
        if self.httpProfile is None or self.clientProfile is None:
            raise InvalidArgumentException

        url = self.httpProfile.protocol + \
              "://" + \
              self.httpProfile.host + \
              ":" + \
              str(self.httpProfile.port) + \
              "/" + \
              self.httpProfile.contextPath + \
              "/" + \
              AICloudConstants.VERSION.lower() + \
              "/" + \
              AICloudConstants.REGION_SERVICE.lower() + \
              "/" + \
              action
        headers = dict()
        headers[AICloudConstants.X_AIC_ACTION] = action
        headers[AICloudConstants.X_AIC_ACCESSKEY] = self.clientProfile.accessKey
        headers["Host"] = self.httpProfile.host + ":" + str(self.httpProfile.port)
        headers[AICloudConstants.X_AIC_SERVICE] = AICloudConstants.REGION_SERVICE

        response = self.client.postForResponse(url, headers, body=request)
        jsonObj = response.json()
        return clazz(jsonObj)

    def DescribeZones(self, request: DescribeZonesRequest) -> DescribeZonesResponse:
        return self.GenericRequest(request.__dict__, "DescribeZones", DescribeZonesResponse)
